# VecGeomConfig.cmake
# -------------------
#
# Config file for the VecGeom package
#
# IMPORTED Targets
# ^^^^^^^^^^^^^^^^
#
# This module defines the following `IMPORTED` targets:
#
# - VecGeom::vecgeom     - Core VecGeom library (static)
# - VecGeom::vgdml       - Present if the GDML parser component was built
# - VecGeom::vecgeomcuda - Present if the CUDA backend was built
#
# Result Variables
# ^^^^^^^^^^^^^^^^
#
# This module defines the following variables either directly, or automatically
# though CMake's find_package command
#
# - VecGeom_FOUND           - True if VecGeom was found
# - VecGeom_BUILTIN_VECCORE - True if VecGeom was built with an internal version on VecCore
# - VecGeom_GDML_FOUND      - True if VecGeom was built with the GDML parser component
# - VecGeom_CUDA_FOUND      - True if VecGeom was built with CUDA support
#
# - VecGeom_scalar_backend_FOUND  - True if the VecGeom backend is scalar
# - VecGeom_vector_backend_FOUND  - True if the VecGeom backend is vector
# - VecGeom_VECTOR_ISA            - String holding the Instruction Set Architecture VecGeom
#                                   was compiled against (e.g. "sse4.2"). The empty string
#                                   if no explicit flag was used.
# - VecGeom_vc_backend_FOUND      - True if the VecGeom vector backend is Vc
#
# Deprecated Variables
# ^^^^^^^^^^^^^^^^^^^^
#
# The following variables are retained for backward compatibility for clients.
# They should no longer be used, and any use migrated to the above IMPORTED targets
# or Result Variables
#
#   VECGEOM_FOUND             - Found the VECGEOM library
#   VECGEOM_INCLUDE_DIRS      - include directories for VecGeom
#   VECGEOM_LIBRARIES         - libraries to link against
#   VECGEOM_DEFINITIONS       - some core flags used to build the vecgeom library and which need to be reused
#                               in projects using VecGeom
#   VECGEOM_INSTALL_DIR       - the top level installation path
#   VECGEOM_EXTERNAL_INCLUDES - external include paths used during vecgeom compilation (for automatic "discovery" of dependencies)
#   VECGEOM_C_COMPILER        - the C compiler used to compile VecGeom
#   VECGEOM_CXX_COMPILER      - the CXX compiler used to compile VecGeom
#


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was VecGeomConfig.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../../../../" ABSOLUTE)

# Use original install prefix when loaded through a "/usr move"
# cross-prefix symbolic link such as /lib -> /usr/lib.
get_filename_component(_realCurr "${CMAKE_CURRENT_LIST_DIR}" REALPATH)
get_filename_component(_realOrig "/usr/lib/mips64el-linux-gnuabi64/cmake/VecGeom" REALPATH)
if(_realCurr STREQUAL _realOrig)
  set(PACKAGE_PREFIX_DIR "/usr")
endif()
unset(_realOrig)
unset(_realCurr)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

# Refind required dependencies
include(CMakeFindDependencyMacro)

# Always VecCore
set(VecGeom_BUILTIN_VECCORE OFF)
if(VecGeom_BUILTIN_VECCORE)
  # Must find our builtin where we expect
  find_dependency(VecCore 0.8.0 NO_DEFAULT_PATH PATHS "" COMPONENTS  )
else()
  find_dependency(VecCore 0.8.0 COMPONENTS )
endif()

# Scalar/Vector Backend, Backend Implementation, and ISA
set(VecGeom_scalar_backend_FOUND True)
set(VecGeom_vc_backend_FOUND )

if(VecGeom_vc_FOUND)
  set(VecGeom_vector_backend_FOUND ON)
endif()

# Take default, but clear if it's "empty" as this is an internal "no flag supplied" case
set(VecGeom_VECTOR_ISA "empty")
if(VecGeom_VECTOR_ISA STREQUAL "empty")
  unset(VecGeom_VECTOR_ISA)
endif()

# Optional CUDA (no dependency?)
set(VecGeom_CUDA_FOUND )
set(VECGEOM_CUDA_ARCH "")

# Optional GDML (Xerces-C)
set(VecGeom_GDML_FOUND ON)
if(VecGeom_GDML_FOUND)
  find_dependency(XercesC)
endif()

# To be reviewed - Geant4 may only be for testing/benchmarking
set(VECGEOM_GEANT4 )

# Include our target definitions
include("${CMAKE_CURRENT_LIST_DIR}/VecGeomTargets.cmake")

# Older variables/defs
set(VECGEOM_C_COMPILER "/usr/bin/cc")
set(VECGEOM_CXX_COMPILER "/usr/bin/c++")
set(VECGEOM_CXX_FLAGS "${CMAKE_CXX_FLAGS}  -faligned-new")
set(VECGEOM_COMPILE_OPTIONS  -faligned-new)

get_filename_component(THIS_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)
set(VECGEOM_INCLUDE_DIR "${THIS_DIR}/../../../../include")
set(VECGEOM_EXTERNAL_INCLUDES "")
set(VECGEOM_INCLUDE_DIRS ${VECGEOM_INCLUDE_DIR} ${VECGEOM_EXTERNAL_INCLUDES})

set(VECGEOM_LIBRARIES VecGeom::vecgeom)
if(TARGET VecGeom::vecgeomcuda)
  list(APPEND VECGEOM_LIBRARIES VecGeom::vecgeomcuda)
endif()
if(TARGET VecGeom::vecgeomcuda_static)
  set(VECGEOM_CUDA_STATIC_LIBRARY VecGeom::vecgeomcuda_static)
endif()

check_required_components(VecGeom)
